@@ -1,12 +1,13 @@
+import io
 import sys
-from typing import List
+from typing import AsyncGenerator, Callable, List
 
+import a2wsgi
 import httpx
 import pytest
-from asgiref.typing import HTTPRequestEvent, HTTPScope
 
-from uvicorn._types import Environ, StartResponse
-from uvicorn.middleware.wsgi import WSGIMiddleware, build_environ
+from uvicorn._types import Environ, HTTPRequestEvent, HTTPScope, StartResponse
+from uvicorn.middleware import wsgi
 
 
 def hello_world(environ: Environ, start_response: StartResponse) -> List[bytes]:
@@ -31,7 +32,7 @@ def echo_body(environ: Environ, start_response: StartResponse) -> List[bytes]:
     return [output]
 
 
-def raise_exception(environ: Environ, start_response: StartResponse) -> RuntimeError:
+def raise_exception(environ: Environ, start_response: StartResponse) -> List[bytes]:
     raise RuntimeError("Something went wrong")
 
 
@@ -49,44 +50,62 @@ def return_exc_info(environ: Environ, start_response: StartResponse) -> List[byt
         return [output]
 
 
-@pytest.mark.asyncio
-async def test_wsgi_get() -> None:
-    app = WSGIMiddleware(hello_world)
+@pytest.fixture(params=[wsgi._WSGIMiddleware, a2wsgi.WSGIMiddleware])
+def wsgi_middleware(request: pytest.FixtureRequest) -> Callable:
+    return request.param
+
+
+@pytest.mark.anyio
+async def test_wsgi_get(wsgi_middleware: Callable) -> None:
+    app = wsgi_middleware(hello_world)
     async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
         response = await client.get("/")
     assert response.status_code == 200
     assert response.text == "Hello World!\n"
 
 
-@pytest.mark.asyncio
-async def test_wsgi_post() -> None:
-    app = WSGIMiddleware(echo_body)
+@pytest.mark.anyio
+async def test_wsgi_post(wsgi_middleware: Callable) -> None:
+    app = wsgi_middleware(echo_body)
     async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
         response = await client.post("/", json={"example": 123})
     assert response.status_code == 200
     assert response.text == '{"example": 123}'
 
 
-@pytest.mark.asyncio
-async def test_wsgi_exception() -> None:
+@pytest.mark.anyio
+async def test_wsgi_put_more_body(wsgi_middleware: Callable) -> None:
+    async def generate_body() -> AsyncGenerator[bytes, None]:
+        for _ in range(1024):
+            yield b"123456789abcdef\n" * 64
+
+    app = wsgi_middleware(echo_body)
+    async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
+        response = await client.put("/", content=generate_body())
+    assert response.status_code == 200
+    assert response.text == "123456789abcdef\n" * 64 * 1024
+
+
+@pytest.mark.anyio
+async def test_wsgi_exception(wsgi_middleware: Callable) -> None:
     # Note that we're testing the WSGI app directly here.
     # The HTTP protocol implementations would catch this error and return 500.
-    app = WSGIMiddleware(raise_exception)
+    app = wsgi_middleware(raise_exception)
     async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
         with pytest.raises(RuntimeError):
             await client.get("/")
 
 
-@pytest.mark.asyncio
-async def test_wsgi_exc_info() -> None:
+@pytest.mark.anyio
+async def test_wsgi_exc_info(wsgi_middleware: Callable) -> None:
     # Note that we're testing the WSGI app directly here.
     # The HTTP protocol implementations would catch this error and return 500.
-    app = WSGIMiddleware(return_exc_info)
+    app = wsgi_middleware(return_exc_info)
     async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
         with pytest.raises(RuntimeError):
             response = await client.get("/")
 
-    app = WSGIMiddleware(return_exc_info)
+    app = wsgi_middleware(return_exc_info)
     transport = httpx.ASGITransport(
         app=app,
         raise_app_exceptions=False,
@@ -100,7 +119,7 @@ async def test_wsgi_exc_info() -> None:
 
 
 def test_build_environ_encoding() -> None:
-    scope: HTTPScope = {
+    scope: "HTTPScope" = {
         "asgi": {"version": "3.0", "spec_version": "2.0"},
         "scheme": "http",
         "raw_path": b"/\xe6\x96\x87",
@@ -115,11 +134,11 @@ def test_build_environ_encoding() -> None:
         "headers": [(b"key", b"value1"), (b"key", b"value2")],
         "extensions": {},
     }
-    message: HTTPRequestEvent = {
+    message: "HTTPRequestEvent" = {
         "type": "http.request",
         "body": b"",
         "more_body": False,
     }
-    environ = build_environ(scope, message, b"")
+    environ = wsgi.build_environ(scope, message, io.BytesIO(b""))
     assert environ["PATH_INFO"] == "/文".encode("utf8").decode("latin-1")
     assert environ["HTTP_KEY"] == "value1,value2"
